
#include <windows.h>  
#include <commctrl.h>
#include "getcount.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "App";
LPCTSTR lpszTitle   = "TreeView_GetCount()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

VOID AddItemsToTree( HWND hTree );

#define IMAGESIZE 16

LPCTSTR lpszData[4] = { "Circle", "Rectangle", "Cross", "Check" };

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HWND hTree = NULL;

   switch( uMsg )
   {
      case WM_CREATE  :
              {
                 HIMAGELIST hImage;

                 InitCommonControls();

                 hImage   = ImageList_Create( IMAGESIZE, IMAGESIZE, 
                                              ILC_COLOR4 | ILC_MASK, 4, 1 ); 
                 hTree = CreateWindowEx( WS_EX_CLIENTEDGE, WC_TREEVIEW, "",
                                         WS_CHILD | WS_BORDER | WS_VISIBLE | 
                                         WS_VSCROLL | TVS_HASLINES | 
                                         TVS_HASBUTTONS | TVS_LINESATROOT |
                                         TVS_DISABLEDRAGDROP,
                                         10, 100, 100, 100,
                                         hWnd,
                                         (HMENU)1,
                                         hInst,
                                         NULL);

                 if ( hTree )
                 {
                    int i;

                    // Associate the image list with the tree view.
                    //.............................................
                    TreeView_SetImageList( hTree, hImage, TVSIL_NORMAL );

                    // Add the images to the tree view image list.
                    //............................................
                    for ( i=0; i<4; i++ )
                       ImageList_AddIcon( hImage, LoadIcon( hInst, lpszData[i] ) );

                    AddItemsToTree( hTree );
                 }
              }
              break;

      case WM_SIZE :
              if ( hTree )
                 MoveWindow( hTree, 0, 0, LOWORD( lParam ), HIWORD( lParam ), FALSE );
              break;

      case WM_INITMENU :
              // Gray out the menu options when they are not appropriate.
              //.........................................................
              if ( TreeView_GetCount( hTree ) < TreeView_GetVisibleCount( hTree ) )
              {
                 EnableMenuItem( (HMENU)wParam, IDM_PGUP, MF_BYCOMMAND | MF_GRAYED );
                 EnableMenuItem( (HMENU)wParam, IDM_PGDOWN, MF_BYCOMMAND | MF_GRAYED );
              }
              else
              {
                 HTREEITEM hCurItem = TreeView_GetSelection( hTree );

                 EnableMenuItem( (HMENU)wParam, IDM_PGUP, MF_BYCOMMAND | 
                                 ( TreeView_GetPrevVisible( hTree, hCurItem ) ?
                                   MF_ENABLED : MF_GRAYED ) );

                 EnableMenuItem( (HMENU)wParam, IDM_PGDOWN, MF_BYCOMMAND | 
                                 ( TreeView_GetNextVisible( hTree, hCurItem ) ?
                                   MF_ENABLED : MF_GRAYED ) );
              }
              break;


      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_PGUP :
                 case IDM_PGDOWN :
                        {
                           HTREEITEM hCurItem, hFirstVis, hNewItem;
                           int       nVisCount;

                           // Retrieve the current state of the tree view.
                           //.............................................
                           nVisCount = TreeView_GetVisibleCount( hTree );
                           hCurItem  = TreeView_GetSelection( hTree );
                           hFirstVis = TreeView_GetFirstVisible( hTree );

                           // Process the page up.
                           //.....................
                           if ( LOWORD( wParam ) == IDM_PGUP )
                           {
                              if ( hCurItem != hFirstVis )
                                 hNewItem = hFirstVis;
                              else
                              {
                                 int nCnt = 0;

                                 hNewItem = hCurItem;
                                 while ( nCnt < nVisCount && 
                                     TreeView_GetPrevVisible( hTree, hNewItem ) )
                                 {
                                    hNewItem = TreeView_GetPrevVisible( hTree, hNewItem );
                                    nCnt++;
                                 }
                              }

                              // Select the new item and make sure it is visible.
                              //.................................................
                              TreeView_SelectSetFirstVisible( hTree, hNewItem );
                              TreeView_SelectItem( hTree, hNewItem );
                           }
                           // Process the Page down.
                           //.......................
                           else
                           {
                              HTREEITEM hTmp;
                              int       nCnt = 1;

                              // Find the last visible item.
                              //............................
                              hTmp = hFirstVis;
                              while( hTmp && nCnt < nVisCount && 
                                     TreeView_GetNextVisible( hTree, hTmp ) )
                              {
                                 hTmp = TreeView_GetNextVisible( hTree, hTmp );
                                 nCnt++;
                              }

                              // Determine the new item.
                              //........................
                              if ( hTmp != hCurItem )
                                 hNewItem = hTmp;
                              else
                              {
                                 nCnt = 1;

                                 hNewItem = hCurItem;

                                 while ( nCnt < nVisCount && 
                                     TreeView_GetNextVisible( hTree, hNewItem ) )
                                 {
                                    hNewItem = TreeView_GetNextVisible( hTree, hNewItem );
                                    nCnt++;
                                 }
                              }

                              // Select the new item and make sure it is visible.
                              //.................................................
                              TreeView_EnsureVisible( hTree, hNewItem );
                              TreeView_SelectItem( hTree, hNewItem );
                           }
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              if ( TreeView_GetImageList( hTree, TVSIL_NORMAL ) )
                 ImageList_Destroy( TreeView_GetImageList( hTree, TVSIL_NORMAL ) );

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


VOID AddItemsToTree( HWND hTree )
{
   int             i;
   char            szTmp[40];
   TV_INSERTSTRUCT tv;

   tv.hInsertAfter = TVI_LAST;
   tv.item.mask    = TVIF_TEXT | TVIF_IMAGE | 
                     TVIF_SELECTEDIMAGE | TVIF_PARAM;

   // Add the items to the tree view.
   //................................
   for ( i=0; i<20; i++ )
   {
      wsprintf( szTmp, "%s %d", lpszData[0], i+1 );

      tv.hParent      = TVI_ROOT;
      tv.item.pszText = szTmp;
      tv.item.iImage  = 0;
      tv.item.iSelectedImage = 0;
      tv.item.lParam         = 0;

      TreeView_InsertItem( hTree, &tv );
   }
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}


